@@ -1,5 +1,5 @@
 from hpogrid._version import __version__
-from quickstats.decorators import *
+from hpogrid.core.decorators import *
 
 import os
 import pathlib
