@@ -381,7 +381,7 @@ class HPOTask(AbstractObject):
         rename_cols = { 'config/{}'.format(hp): hp for hp in self.hyperparameters}
         rename_cols['time_total_s'] = 'time_s'
         df = df.rename(columns=rename_cols)
-        cols_to_save = ['time_s'] + self.hyperparameters
+        cols_to_save = ['time_s', 'timestamp'] + self.hyperparameters 
         cols_to_save.append(self.metric)
         if self.extra_metrics:
             cols_to_save += self.extra_metrics
@@ -417,6 +417,7 @@ class HPOTask(AbstractObject):
         }
          
         df = self.get_processed_df(df)
+        df = df.sort_values("timestamp", ascending=True)
         df = df.transpose()
         
         summary['result'] = df.to_dict()
