@@ -16,16 +16,16 @@ class GridGenerator(Generator):
         self.labels = labels
         return search_points
 
-    def tell(self, point:Dict, value):
+    def _tell(self, searcher, point:Dict, value):
         # search point values
         value = tuple([point[label] for label in self.labels])
-        if value in self.searcher:
-            self.searcher.remove(value)
+        if value in searcher:
+            searcher.remove(value)
 
-    def ask(self, n_points:int =None):
+    def _ask(self, searcher, n_points:int =None):
         points = []
-        if n_points > len(self.searcher):
-            n_points = len(self.searcher)
+        if n_points > len(searcher):
+            n_points = len(searcher)
         for i in range(n_points):
-            points.append({ k:v for k,v in zip(self.labels, self.searcher[i])})
+            points.append({ k:v for k,v in zip(self.labels, searcher[i])})
         return points
\ No newline at end of file
