@@ -20,15 +20,15 @@ class NeverGradGenerator(Generator):
                 parametrization=search_space, budget=self.kDefaultBudget)
         return searcher
 
-    def ask(self, n_points:int = None):
+    def _ask(self, searcher, n_points:int = None):
         points = []
         for _ in range(n_points):
-            point = self.searcher.ask().kwargs
+            point = searcher.ask().kwargs
             points.append(copy.deepcopy(point))
         return points
 
-    def tell(self, point:Dict, value):
+    def _tell(self, searcher, point:Dict, value):
         value = self._to_metric_values(value)
-        self.searcher.suggest(**point)
-        candidate = self.searcher.ask()
-        self.searcher.tell(candidate, self.signature * value)
+        searcher.suggest(**point)
+        candidate = searcher.ask()
+        searcher.tell(candidate, self.signature * value)
