@@ -17,9 +17,9 @@ class SkOptGenerator(Generator):
 
     def _ask(self, searcher, n_points:int = None):
         points = []
-        for _ in range(n_points):
-            point = searcher.ask()
-            point = dict(zip(self.labels, point))
+        points_raw = searcher.ask(n_points)
+        for point_raw in points_raw:
+            point = dict(zip(self.labels, point_raw))
             points.append(copy.deepcopy(point))
         return points
 
