@@ -1,10 +1,10 @@
 import math
-from past.utils import old_div
-
 import numpy as np
 
 from .base_space import BaseSpace
 
+from hpogrid.utils.helper import old_div
+
 class PBTSpace(BaseSpace):
 
     def __init__(self, search_space = None):
